<?php
namespace Cashbene\GatewayMagento\Controller\Index;

use Magento\Framework\App\ActionInterface;
use Cashbene\Core\Dto\MobileNumber;
use Cashbene\Core\Dto\Register\RegisterUser;
use Cashbene\Core\Exception\HttpExceptionInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Cashbene\GatewayMagento\Controller\CashbeneAbstractRestController;

class Registration extends CashbeneAbstractRestController implements ActionInterface
{
    const POST_ACTIONS = ['smsVerificationCode', 'checkPinCode', 'individualSignUp'];
    const GET_ACTIONS = ['emailExist'];

    protected function processGetRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'emailExist':
                return $this->emailExist($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see RegisterContext::emailExist()
     */
    private function emailExist(Request $request) {
        try {
            $data = self::$cashbeneGateway->RegisterContext()->emailExist($request->get('email'));
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }

    protected function processPostRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'smsVerificationCode':
                return $this->generateSmsVerificationCode($request);
            case 'checkPinCode':
                return $this->checkPinCode($request);
            case 'individualSignUp':
                return $this->individualSignUp($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see RegisterContext::generateSmsVerificationCode()
     */
    private function generateSmsVerificationCode(Request $request) {
        $params = json_decode($request->getContent(), true);
        $mobileNumberDto = self::$serializer->denormalize(
            $params['mobile'], MobileNumber::class, 'array'
        );

        try {
            self::$cashbeneGateway->RegisterContext()->generateSmsVerificationCode($mobileNumberDto);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success(null, 201);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see RegisterContext::checkPinCode()
     */
    private function checkPinCode(Request $request) {
        try {
            $params = json_decode($request->getContent(), true);
            self::$cashbeneGateway->RegisterContext()->checkPinCode($params['password']);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success();
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see RegisterContext::signUp()
     */
    private function individualSignUp(Request $request) {
        $params = json_decode($request->getContent(), true);
        $registerUserDto = self::$serializer->denormalize(
            $params, RegisterUser::class, 'array'
        );

        try {
            self::$cashbeneGateway->RegisterContext()->signUp($registerUserDto);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success(null, 201);
    }

    protected function processPutRequest(string $action, Request $request)
    {
        // TODO: Implement processPutRequest() method.
    }

    protected function processDeleteRequest(string $action, Request $request)
    {
        // TODO: Implement processDeleteRequest() method.
    }

}
