<?php

namespace Cashbene\GatewayMagento\Helper;

use Cashbene\GatewayMagento\Service\CheckoutService;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Quote\Model\Quote\Address\Rate;
use Cashbene\GatewayMagento\Utils\Shop;
use Magento\Quote\Model\Quote;
use Cashbene\PaymentMagento\Model\Payment\PaymentMethod;
use Magento\Store\Model\StoreManagerInterface;
use \Magento\Quote\Model\QuoteFactory;
use \Magento\Quote\Api\CartRepositoryInterface;
use \Magento\Quote\Api\CartManagementInterface;
use \Magento\Customer\Model\CustomerFactory;
use \Magento\Customer\Api\CustomerRepositoryInterface;
use \Magento\Quote\Model\Quote\Payment;
use \Magento\Sales\Model\Order;
use \Magento\Sales\Api\OrderRepositoryInterface;
use \Magento\Customer\Api\Data\CustomerInterfaceFactory;

class Data
{
    public function __construct(
        StoreManagerInterface $storeManager,
        QuoteFactory $quote,
        CartRepositoryInterface $cartRepositoryInterface,
        CartManagementInterface $quoteManagement,
        CustomerFactory $customerFactory,
        CustomerInterfaceFactory $customerFactoryInterface,
        CustomerRepositoryInterface $customerRepository,
        ProductRepositoryInterface $productRepository,
        Rate $rate,
        Payment $payment,
        Order $order,
        OrderRepositoryInterface $orderRepository
    ) {
        $this->_storeManager = $storeManager;
        $this->quote = $quote;
        $this->cartRepositoryInterface = $cartRepositoryInterface;
        $this->quoteManagement = $quoteManagement;
        $this->customerFactory = $customerFactory;
        $this->customerFactoryInterface = $customerFactoryInterface;
        $this->customerRepository = $customerRepository;
        $this->productRepository = $productRepository;
        $this->rate = $rate;
        $this->payment = $payment;
        $this->order = $order;
        $this->orderRepository = $orderRepository;
    }

    /**
     * Create Order On Your Store
     *
     * @param array $orderData
     * @return array
     *
     */
    public function createMageOrder($orderData, $cart = null) {

        if ($cart != null) {
            $quote = $this->prepareQuote($orderData, $cart);
        } else {
            $quote = $this->prepareQuote($orderData);
        }

        $orderId = $this->quoteManagement->placeOrder($quote->getId());
        $order = $this->orderRepository->get($orderId);

        if ($cart != null) {
            $order->setShippingAmount($orderData['shipping_price']);
            $order->setBaseShippingAmount($orderData['shipping_price']);
            $order->setShippingInclTax($orderData['shipping_price']);
            $order->setBaseShippingInclTax($orderData['shipping_price']);

            $baseGrandTotal = $order->getBaseSubtotal() + $order->getBaseShippingAmount();
            $grandTotal = $order->getSubtotalInclTax() + $order->getBaseShippingInclTax();
            $order->setBaseGrandTotal($baseGrandTotal);
            $order->setGrandTotal($grandTotal);
        }

        $this->orderRepository->save($order);

        $order->setEmailSent(0);
        // todo check if mails are send, or need to save here.

        if ($order->getEntityId()) {
            return $order;
        } else {
            $result = ['error' => 1, 'msg'=>'Your custom message'];
        }

        return $result;
    }

    public function calculateVatFromPrices($priceWithVat, $priceNoVat) {
        return ($priceWithVat - $priceNoVat) / $priceNoVat * 100;
    }

    /**
     * @param $orderData
     * @return CustomerInterface
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Magento\Framework\Exception\State\InputMismatchException
     */
    private function getCustomer($orderData) {
        $store = $this->_storeManager->getStore();
        $websiteId = $store->getWebsiteId();
        $customer = $this->customerFactory->create();
        $customer->setWebsiteId($websiteId);
        $customer->loadByEmail($orderData['email']);// load customet by email address

        if (!$customer->getEntityId()){
            //If not avilable then create this customer
            $customer = $this->customerFactoryInterface->create();
            $customer
                ->setWebsiteId($websiteId)
                ->setStoreId($store->getId())
                ->setFirstname($orderData['shipping_address']['firstname'])
                ->setLastname($orderData['shipping_address']['lastname'])
                ->setEmail($orderData['email'])
            ;

            $customer = $this->customerRepository->save($customer);
        }

        return $this->customerRepository->getById($customer->getId());
    }

    /**
     * @param \Magento\Quote\Model\Quote $quote
     * @param $items
     */
    public function addProducts($quote, $items): void
    {
        foreach($items as $item) {
            $product = $this->productRepository->getById($item['product_id']);

            $productVariation = null;
            $attrCodes = [];

            if ($product->getTypeId() == 'configurable' && $item['product_id'] !== $item['product_variation']) {
                $productVariation = $this->productRepository->getById($item['product_variation']);
                $attributes = Shop::getAttributesArray($product, $productVariation);

                foreach ($attributes as $attribute) {
                    $attrCodes[$attribute['attr_id']] = $attribute['value_id'];
                }
            }

            $objParam = new \Magento\Framework\DataObject(
                [
                    'product' => $product, //configurable product id
                    'selected_configurable_option' => $productVariation, //selected simple product id
                    'qty' => isset($item['qty']) ? $item['qty'] : 1,
                    'super_attribute' => $attrCodes
                ]
            );

            $quote->addProduct($product, $objParam);
        }
    }

    /**
     * @param Quote $quote
     * @param array $orderData
     */
    private function setShippingMethod(Quote $quote, array $orderData): void
    {
        $this->rate
            ->setCode($orderData['shipping_method'])
            ->setPrice($orderData['shipping_price'])
            ->setCarrierTitle($orderData['shipping_title'])
        ;

        $quote->getShippingAddress()
            ->setShippingMethod($orderData['shipping_method'])
            ->addShippingRate($this->rate)
            ->setShippingAmount($orderData['shipping_price'])
            ->setBaseShippingAmount($orderData['shipping_price'])
        ;
    }

    /**
     * @param array $orderData
     * @return mixed
     */
    public function prepareQuote(array $orderData, $cart = null)
    {
        $store = $this->_storeManager->getStore();
        if ($cart == null) {
            $quote = $this->quote->create(); //Create object of quote

            // add products
            $this->addProducts($quote, $orderData['items']);
        } else {
            $quote = $cart;
            $quote->getShippingAddress()->removeAllShippingRates();
        }

        $quote->setStore($store); //set store for which you create quote
        $quote->setCurrency();

        if (isset($orderData['email'])) {
            $quote->assignCustomer($this->getCustomer($orderData)); //Assign quote to customer
        }

        // Set Address to quote
        $quote->getBillingAddress()->addData($orderData['billingAddress']);
        $quote->getShippingAddress()->addData($orderData['shipping_address']);

        // Shipping & Handling Information
        $this->setShippingMethod($quote, $orderData);

        $quote->setInventoryProcessed(false); //not effetc inventory
        $this->cartRepositoryInterface->save($quote);

        // Set Sales Order Payment
        $quote->getPayment()->importData(['method' => PaymentMethod::PAYMENT_NAME]);

        // Collect Totals & Save Quote
        $quote->collectTotals();

        $this->cartRepositoryInterface->save($quote);

        // Create Order From Quote
        $quote = $this->cartRepositoryInterface->get($quote->getId());

        return $quote;
    }
}
