<?php
namespace Cashbene\GatewayMagento\Model\Config;

use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Value;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;
use Magento\Framework\Serialize\SerializerInterface;
use Cashbene\GatewayMagento\Service\ConfigurationService;

/**
 * Backend for serialized array data
 */
class ShippingFields extends Value
{
    /**
     * Json Serializer
     *
     * @var SerializerInterface
     */
    protected $serializer;
    /** @var ConfigurationService */
    private $configurationService;
    /** @var bool */
    private static $isCredentialFieldChange = false;

    /**
     * ShippingMethods constructor
     *
     * @param SerializerInterface $serializer
     * @param Context $context
     * @param Registry $registry
     * @param ScopeConfigInterface $config
     * @param TypeListInterface $cacheTypeList
     * @param AbstractResource|null $resource
     * @param AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        SerializerInterface $serializer,
        Context $context,
        Registry $registry,
        ScopeConfigInterface $config,
	TypeListInterface $cacheTypeList,
	ConfigurationService $configurationService,
        AbstractResource $resource = null,
        AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->serializer = $serializer;
        $this->configurationService = $configurationService;
        parent::__construct($context, $registry, $config, $cacheTypeList, $resource, $resourceCollection, $data);
    }

    /**
     * @var bool $value
     */
    public static function setIsCredentialFieldChange($value)
    {
        self::$isCredentialFieldChange = $value;
    }

    /**
     * Prepare data before save
     *
     * @return void
     */
    public function beforeSave()
    {
        $shippings = $this->getData()['groups']['general']['fields'][$this->getData()['field']];

        $array = [];
        foreach ($shippings as $id => $shipping) {
            $array[$id] = $shipping['value'];
        }

        $encodedValue = $this->serializer->serialize($array);
        $this->setValue($encodedValue);

        if (empty(array_filter($array))) {
            $this->configurationService->setConfigValue('show_cashbene_box', null);
        } elseif (self::$isCredentialFieldChange === false) {
            $this->configurationService->setConfigValue('show_cashbene_box', true);
        }
    }
}
