<?php

namespace Cashbene\GatewayMagento\Model\Config\Source;

use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Data\OptionSourceInterface;
use Magento\Cms\Api\PageRepositoryInterface;
use Magento\Framework\Api\Filter;
use Magento\Framework\Api\Search\FilterGroup;
use Magento\Framework\Exception\LocalizedException;

class Pages implements OptionSourceInterface
{
    /**
     * @var PageRepositoryInterface
     */
    private $_cmsPage;

    public function __construct(
        PageRepositoryInterface $pageRepository,
        SearchCriteriaInterface $searchCriteria,
        Filter $filter,
        FilterGroup $filterGroup
    ) {
        $this->_cmsPage = $pageRepository;
        $this->searchCriteria = $searchCriteria;
        $this->filter = $filter;
        $this->filterGroup = $filterGroup;
    }

    /**
     * @return array
     * @throws LocalizedException
     */
    public function toOptionArray(): array
    {
        $pages = [
            ['value' => '', 'label' => '-']
        ];
        foreach($this->_cmsPage->getList($this->_getSearchCriteria())->getItems() as $page) {
            $pages[] = [
                'value' => $page->getId(),
                'label' => $page->getTitle()
            ];
        }
        return $pages;
    }

    protected function _getSearchCriteria(): SearchCriteriaInterface
    {
        $filter = $this->filter
            ->setField('is_active')
            ->setValue(1)
            ->setConditionType('eq');

        $filterGroup = $this->filterGroup->setFilters([$filter]);
        return $this->searchCriteria->setFilterGroups([$filterGroup]);
    }
}
