<?php

namespace Cashbene\GatewayMagento\Model\FormField;

use Cashbene\GatewayMagento\Service\GatewayService;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Backend\Block\Template\Context;
use \Magento\Framework\App\Config\ScopeConfigInterface;
use \Magento\Shipping\Model\Config;
use \Magento\Config\Block\System\Config\Form\Field;

class ShippingFields extends Field
{

    protected $scopeConfig;
    protected $shipconfig;

    /** @var \Cashbene\Core\Dto\Shop\ShopShippingMethod[] */
    private $shippings;

    public function __construct(
        Context $context,
        GatewayService $gatewayService,
        ScopeConfigInterface $scopeConfig,
        Config $shipconfig,
	SerializerInterface $serializer,
	array $data = []
    )
    {
        parent::__construct($context, $data);
        $this->setTemplate('widget/custom-template.phtml');
        $this->gateway = $gatewayService->getGateway();
        $this->shipconfig = $shipconfig;
        $this->scopeConfig = $scopeConfig;
        $this->serializer = $serializer;
    }

    public function render(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        if (!$this->gateway->isSetMerchantCredentials()) {
            return null;
        }

        try {
            $this->shippings = $this->gateway->shopContext()->getMerchantShippingMethods();
        } catch (\Exception $e) {
            return null;
        }

        $html = '<td class="label"><label for="' .
            $element->getHtmlId() . '"><span' .
            $this->_renderScopeLabel($element) . '>' .
            $element->getLabel() .
            '</span></label></td>';
        $html .= $this->_renderValue($element);

        return $this->_decorateRowHtml($element, $html);
    }

    public function _getElementHtml(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        $values = json_decode($element->getData('value') ?? "", true);

        $html = '';
        foreach ($this->shippings as $shipping) {
            $html .= $this->getShippingMethod($shipping, $values[$shipping->id] ?? null);
        }

        return $html;
    }

    public function _toHtml()
    {
        return parent::_toHtml();
    }

    protected function getShippingMethod($shipping, $value = null)
    {
        if ($shipping->active) {
            $html = '<tr id="row_cashbene_general_' . $shipping->id . '">
                <td class="label">
                    <label for="cashbene_general_' . $shipping->id . '">
                    <span>' . $shipping->name . '</span>
                    </label>
                </td>
                <td class="value">
                    <select id="cashbene_general_' . $shipping->id . '" name="groups[general][fields][shipping_method][' . $shipping->id . '][value]" class=" select admin__control-select">';

                    $html .= '<option value="">-</option>';
            foreach ($this->prepareMagentoShippingArray() as $id => $name) {
                $selected = $id === $value ? 'selected="selected"':'';
                $html .= '<option value="'. $id .'" id="'. $id .'"  '.$selected.'>'. $name .'</option>';
            }

            $html .= '</select>
                </td>

                </tr>';
            return $html;
        }
        return;
    }

    public function prepareMagentoShippingArray() {
        $shippingArray = [];

        foreach ($this->getMagentoShippingMethods() as $shippingMethod) {
            foreach ($shippingMethod['value'] as $method) {
                $shippingArray[$method['value']] = $shippingMethod['label'] . ' - ' . $method['label'];
            }
        }
        return $shippingArray;
    }

    public function getMagentoShippingMethods() {
        $activeCarriers = $this->shipconfig->getActiveCarriers();

        foreach($activeCarriers as $carrierCode => $carrierModel) {
            $options = [];
            $carrierModel->getCarrierCode();

            if ($carrierMethods = $carrierModel->getAllowedMethods()) {
                foreach ($carrierMethods as $methodCode => $method) {
                    $code = $carrierCode . '_' . $methodCode;
                    $options[] = ['value' => $code, 'label' => $method];
                }
                $carrierTitle = $this->scopeConfig
                    ->getValue('carriers/'.$carrierCode.'/title');
            }

            $methods[] = ['value' => $options, 'label' => $carrierTitle];
        }

        return $methods;
    }
}
