<?php

namespace Cashbene\GatewayMagento\Service;

use Magento\Framework\App\Config\ScopeConfigInterface;
//use Magento\Framework\Locale\Resolver;
use Magento\Framework\Locale\ResolverInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Cashbene\Core\Utils\Configuration as CashbeneConfiguration;

class ConfigurationService
{
    /** @var array $options */
    private $options;

    /** @var array $shippingMethods */
    private $shippingMethods;

    /** @var ScopeConfigInterface */
    private $scopeConfig;

    /** @var Resolver */
    private $resolver;

    /** @var WriterInterface */
    private $writer;

    const SHIPPING_METHOD_KEY = "PAYMENT_GATEWAY_SHIPPING_METHODS_";

    public function __construct(ScopeConfigInterface $scopeConfig, ResolverInterface $resolver, WriterInterface $writer)
    {
        $this->scopeConfig = $scopeConfig;
        $this->resolver = $resolver;
        $this->writer = $writer;
        $this->reloadConfigurationSettings();
    }

    public function getConfigValue($value, $path = 'cashbene/general')
    {
        return $this->scopeConfig->getValue("{$path}/{$value}", ScopeInterface::SCOPE_STORE);
    }

    public function setConfigValue($key, $value, $path = 'cashbene/general')
    {
        $this->writer->save("{$path}/{$key}", $value);
    }

    public function reloadConfigurationSettings($path = 'cashbene/general')
    {
        $this->shippingMethods = $this->prepareShippingMethods(
            $this->scopeConfig->getValue("{$path}/shipping_methods", ScopeInterface::SCOPE_STORE)
            ?? []
        );

        $this->options = [
            'environment'   => $this->getConfigValue("environment"),
            'merchant_id'   => $this->getConfigValue("merchant_id"),
            'client_id'     => $this->getConfigValue("client_id"),
            'client_secret' => $this->getConfigValue("client_secret"),
            'secret_key'    => $this->getConfigValue("secret_key"),
            'language'      => $this->resolver->getLocale()
        ];
    }

    private function prepareShippingMethods(array $shippingMethods): array
    {
        $output = [];
        foreach ($shippingMethods as $key => $shippingMethod) {
            $output[str_replace(self::SHIPPING_METHOD_KEY, '', $key)] = $shippingMethod;
        }

        return $output;
    }

    public function getConfiguration(): CashbeneConfiguration
    {
        $configuration = [
            'merchant_id'   => null,
            'client_id'     => null,
            'client_secret' => null,
            'secret_key'    => null,
            'environment'   => null,
            'language'      => 'pl_PL'
        ];

        $configuration = array_merge($configuration, $this->options ?: []);

        return new CashbeneConfiguration($configuration);
    }

    public function getShippingMethods(): array
    {
        return $this->shippingMethods;
    }
}
