<?php

namespace Cashbene\GatewayMagento\Service;

use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\ResourceModel\Order\Status\Collection as OrderStatusCollection;

class OrderStatusService
{
    public function __construct(
        OrderStatusCollection $orderStatusCollection,
        OrderRepositoryInterface $orderRepository
    )
    {
        $this->orderStatusCollection = $orderStatusCollection;
        $this->orderRepository = $orderRepository;
    }

    public function getAllOrderStatus() {
        return $this->orderStatusCollection->toOptionArray();
    }

    public function getAllOrderStatusKeys() {
        $statusArray = [];
        foreach ($this->orderStatusCollection->toOptionArray() as $status) {
            $statusArray[$status['value']] = $status['value'];
        }

        return $statusArray;
    }

    public function getOrder($id) {
        $order = $this->orderRepository->get($id);
        return $order;
    }

    public function orderSetStatus($orderId, $status) {
        if ($this->checkStatusIsAvailable($status)) {
            $order = $this->orderRepository->get($orderId);
            $order->setStatus($status);
            $order->setState($status);
            $this->orderRepository->save($order);
        } else {
            throw new \Exception('Status not available.');
        }
    }

    public function checkStatusIsAvailable($statusToCheck) {
        foreach ($this->getAllOrderStatus() as $status) {
            if ($status['value'] === $statusToCheck) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param string $status
     * @return false|string
     */
    public function getMagentoStatusByCashbeneStatus(string $status)
    {
        switch ($status) {
            case "SUCCESSFUL":
                return $this->checkStatusIsAvailable('complete') ? 'complete' : false;
            case "FAILED":
                return $this->checkStatusIsAvailable('canceled') ? 'canceled' : false;
            default:
                return false;
        }
    }
}
