<?php

namespace Cashbene\GatewayMagento\Service;

use Magento\Cms\Api\Data\PageInterface;
use Magento\Cms\Api\PageRepositoryInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\ScopeInterface;
use Magento\Framework\UrlInterface;
use Magento\Framework\ObjectManagerInterface;

class RegulationService {

    /** @var ConfigurationService */
    private $configurationService;

    public function __construct(
        ConfigurationService $configurationService,
        UrlInterface $urlInterface,
        ObjectManagerInterface $objectManager
    )
    {
        $this->_urlBuilder = $urlInterface;
        $this->configurationService = $configurationService;
        $this->objectManager = $objectManager;
    }

    /**
     * Stream pdf file to browser.
     * Can't be escaped because it's a pdf file. It's loaded from trusted external api.
     *
     * @param $pdfContent
     * @param $name
     */
    static public function decodePdfBase64ToFile($pdfContent, $name) {
        $pdfToStream = base64_decode($pdfContent);

        header("Content-type: application/octet-stream");
        header("Content-Type: application/pdf");
        header("Content-Disposition: inline; filename=$name.pdf");
        echo $pdfToStream;
    }

    /**
     * @param string $key
     * @return null|array
     */
    public function getMerchantTerm(string $key)
    {
        $pageRepository = $this->objectManager->get(PageRepositoryInterface::class);
        try {
            /**
             * @var $page PageInterface
             */
            $page = $pageRepository->getById($this->configurationService->getConfigValue($key));

        } catch (NoSuchEntityException $exception) {
            return null;
        }

        $pageUrl = $this->_urlBuilder->getUrl(null, ['_direct' => $page->getIdentifier()]);

        return [
            'id' => intval($page->getId()),
            'title' => $page->getTitle(),
            'url' => $pageUrl
        ];
    }
}
